@@ -1,8 +1,6 @@
 from repoze.formapi.py24 import defaultdict
 from repoze.formapi.error import Errors
 
-MISSING = object()
-
 def marshall(params, fields):
     """marshall(params, fields) -> data, errors
 
@@ -67,16 +65,23 @@ def marshall(params, fields):
         True
 
     Note that the ``data`` and ``errors`` dictionaries provide a
-    default value of ``None`` for missing entries.
+    default value of ``missing`` for missing entries.
+
+        >>> from repoze.formapi.marshalling import missing
+
+    It evaluates to false.
+
+        >>> bool(missing)
+        False
 
         >>> data, errors = marshall((), fields)
 
         >>> 'age' in data['user']
         False
-        
-        >>> data['user']['age'] is None
+
+        >>> data['user']['age'] is missing
         True
-        
+
     The fields structure may end in a list of a simple type.
 
         >>> fields = {
@@ -90,12 +95,12 @@ def marshall(params, fields):
         >>> params = (
         ...     ("user.friends", "stefan"),
         ...     ("user.friends", "malthe"))
-        
+
         >>> data, errors = marshall(params, fields)
-        
+
     As expected, we get a list. Note that this list items appear in
     the order they appear in the reqeuest parameters.
-    
+
         >>> data['user']['friends']
         ['stefan', 'malthe']
 
@@ -104,11 +109,11 @@ def marshall(params, fields):
         >>> fields = {
         ...     "points": (int,)
         ...     }
-        
+
         >>> params = (("points", 42), ("points", 10))
-        
+
         >>> data, errors = marshall(params, fields)
-        
+
         >>> data['points']
         (42, 10)
 
@@ -140,15 +145,61 @@ def marshall(params, fields):
         >>> params = (
         ...     ("user.stefan.name", "Stefan Eletzhofer"),
         ...     ("user.malthe.name", "Malthe Borch"))
-        
+
         >>> data, errors = marshall(params, fields)
-        
+
     As expected, we get a list. Note that this list items appear in
     the order they appear in the reqeuest parameters.
-    
+
         >>> data['user']['stefan']['name']
         'Stefan Eletzhofer'
-    
+
+    If a trivial input results in a conversion error, the value is set
+    to ``None``.
+
+        >>> fields = {
+        ...     'user': str,
+        ...     'age': int
+        ...     }
+
+        >>> params = (
+        ...     ('user', ''),
+        ...     ('age', ''))
+
+        >>> data, errors = marshall(params, fields)
+
+     Empty strings are valid for string fields.
+
+        >>> data['user']
+        ''
+
+     Empty strings are trivial for integer fields; it's regarded as a
+     non-input.
+
+        >>> data['age']
+
+     If the field is required, an empty string is an invalid input for
+     a number. Invalid inputs are preserved as-is for required fields.
+
+        >>> from repoze.formapi.marshalling import required
+        >>> fields['age'] = required(int)
+        >>> data, errors = marshall(params, fields)
+
+        >>> data['age'] is None
+        True
+
+        >>> bool(errors['age'])
+        True
+
+     The same goes for non-trivial invalid inputs.
+
+        >>> data, errors = marshall((('age', 'ten'),), fields)
+        >>> data['age']
+        'ten'
+
+        >>> bool(errors['age'])
+        True
+
     """
 
     data = Marshaller(fields)
@@ -185,7 +236,8 @@ class Marshaller(object):
     resolved; if not set, an empty value is returned.
 
         >>> from repoze.formapi.marshalling import Marshaller
-        
+        >>> from repoze.formapi.marshalling import missing
+
         >>> marshaller = Marshaller({
         ...     'name': str,
         ...     'users': {
@@ -197,18 +249,18 @@ class Marshaller(object):
         ...     })
 
     We'll first demonstrate that valid keys return an empty value.
-    
-        >>> marshaller['name'] is None
+
+        >>> marshaller['name'] is missing
         True
-        
-        >>> marshaller['users', 'foo', 'username'] is None
+
+        >>> marshaller['users', 'foo', 'username'] is missing
         True
-        
+
         >>> marshaller['users', 'foo', 'groups']
         []
 
     An exception is raised if an invalid key is tried.
-    
+
         >>> marshaller['users', 'foo', 'bar']
         Traceback (most recent call last):
          ...
@@ -226,15 +278,15 @@ class Marshaller(object):
         >>> marshaller['users', 'foo', 'username'] = 'foo'
 
     Sequences append on assignments.
-        
+
         >>> marshaller['users', 'foo', 'groups'] = 'foo'
         >>> marshaller['users', 'foo', 'groups'] = 'bar'
 
     As expected, we can retrieve the values in exchange for the keys.
-        
+
         >>> marshaller['name']
         'Foo'
-        
+
         >>> marshaller['users', 'foo', 'username']
         'foo'
 
@@ -248,17 +300,17 @@ class Marshaller(object):
 
         >>> marshaller['users', 'foo', 'groups'] = ['bar']
         >>> marshaller['users', 'foo', 'groups']
-        ['bar']    
+        ['bar']
 
     Again, an exception is raised if an invalid key is tried.
-    
+
         >>> marshaller['users', 'foo', 'bar'] = 'baz'
         Traceback (most recent call last):
          ...
         KeyError: 'bar'
 
     Values are attempted coerced to the field type.
-    
+
         >>> marshaller['users', 'foo', 'id'] = '1'
         >>> marshaller['users', 'foo', 'id']
         1
@@ -273,13 +325,12 @@ class Marshaller(object):
     We may always assign ``None``.
 
         >>> marshaller['users', 'foo', 'id'] = None
-        
+
     The truth-value of a non-empty ``Marshaller`` instance is ``True``.
 
         >>> bool(marshaller)
         True
 
-
     If the path is not exhausted, a new marshaller instance is
     returned, which curries the path onto new requests.
 
@@ -293,7 +344,7 @@ class Marshaller(object):
 
         >>> tuple(sorted(marshaller['users']['foo']))
         ('groups', 'id', 'username')
-        
+
     """
 
     def __init__(self, fields, data=None, path=(), coerce=True):
@@ -310,8 +361,8 @@ class Marshaller(object):
         if not isinstance(path, tuple):
             path = (path,)
         path = self.path + tuple(path)
-        value = self.data.get(path, MISSING)
-        if value is not MISSING:
+        value = self.data.get(path, missing)
+        if value is not missing:
             return value
 
         # verify path; we want to raise an exception if the path does
@@ -327,6 +378,8 @@ class Marshaller(object):
         if isinstance(data_type, dict):
             return Marshaller(self.fields, self.data, path, self.coerce)
 
+        return missing
+
     def __setitem__(self, path, value):
         if not isinstance(path, tuple):
             path = (path,)
@@ -365,6 +418,17 @@ class Marshaller(object):
                     value = data_type(value)
                 except:
                     error = True
+                    if not value:
+                        value = None
+
+                # if the returned value does not conform to the type,
+                # raise an exception and set the error flag
+                if not error and not isinstance(value, data_type):
+                    try:
+                        raise ValueError(value)
+                    except:
+                        error = True
+
             self.data[key] = value
 
         if value is not None:
@@ -405,7 +469,7 @@ class Marshaller(object):
                 if key in data:
                     data = data[key]
                 else:
-                    data[key] = defaultdict(lambda: None)
+                    data[key] = defaultdict(lambda: missing)
                     data = data[key]
             data[path[-1]] = value
         return _data
@@ -432,6 +496,14 @@ class Marshaller(object):
             fields = fields[value]
         return fields
 
+def required(cls, msg="Required field"):
+    class required(cls):
+        def __new__(base, value):
+            if not value:
+                raise ValueError(msg)
+            return cls.__new__(base, value)
+    return required
+
 class defaultdict(defaultdict):
     __doc__ = defaultdict.__doc__
 
@@ -440,4 +512,10 @@ class defaultdict(defaultdict):
         # detail that this is a ``defaultdict`` object.
         return dict.__repr__(self)
 
+class Missing(object):
+    def __nonzero__(self):
+        return False
+
+missing = Missing()
+
 # vim: set ft=python ts=4 sw=4 expandtab : 
